#include "grblsettingsdialog.h"
#include "mainwindow.h"
#include "ui_grblsettingsdialog.h"
#include <QDebug>

#define PARA_IDX_MAX 132

QColor ChangedValueColor(255,114,10);

GrblSettingsDialog::GrblSettingsDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::GrblSettingsDialog)
{
    ui->setupUi(this);
    ui->tableWidgetParameter->verticalHeader()->hide();
    ui->tableWidgetParameter->setShowGrid(true);
    connect(ui->tableWidgetParameter,SIGNAL(cellChanged(int,int)),this,SLOT(cellChanged(int,int)));
}

GrblSettingsDialog::~GrblSettingsDialog()
{
    delete ui;
}

void GrblSettingsDialog::on_pushButtonClose_clicked()
{
    this->hide();
}

void GrblSettingsDialog::on_pushButtonApply_clicked()
{
    MainWindow * TheMainWindow = 0;
    if(parent()){
      TheMainWindow = static_cast<MainWindow*>(parent());
    }else{
        return;
    }

    for(int i = 0; i < ui->tableWidgetParameter->rowCount(); i++){

        if(ui->tableWidgetParameter->item(i,1)->backgroundColor()==ChangedValueColor){

            QString IdxStr = ui->tableWidgetParameter->item(i,0)->text();
            QString ValStr = ui->tableWidgetParameter->item(i,1)->text();

            if(TheMainWindow){
                TheMainWindow->TryToSendLineAndAddToConsoleText(QString("$%1=%2").arg(IdxStr).arg(ValStr));
            }
        }
    }
}

void GrblSettingsDialog::cellChanged(int row, int column)
{
    disconnect(ui->tableWidgetParameter,SIGNAL(cellChanged(int,int)),this,SLOT(cellChanged(int,int)));

    bool ok = false;
    qDebug()<<"Cell Changed"<<row<<column;
    QString NewValStr = ui->tableWidgetParameter->item(row,column)->text();
    double NewValue = NewValStr.toDouble(&ok);

    int i = GetListIndexOfParameterWithIdx(ui->tableWidgetParameter->item(row,0)->text().toInt());
    if(i != -1){
        double OldValue = ParametersList.at(i).Value;

        if(ok){
            if(NewValue != OldValue){
                ui->tableWidgetParameter->item(row,column)->setBackgroundColor(ChangedValueColor);
            }
        }else{
             ui->tableWidgetParameter->item(row,column)->setText(QString::number(OldValue));
        }
    }

    connect(ui->tableWidgetParameter,SIGNAL(cellChanged(int,int)),this,SLOT(cellChanged(int,int)));

}

int GrblSettingsDialog::GetListIndexOfParameterWithIdx(int Idx)
{
    for(int i = 0; i < ParametersList.size(); i++){
        if(ParametersList.at(i).Index == Idx){
            return i;
        }
    }
    return -1;
}


void GrblSettingsDialog::AddParameterToList(GrblParameter NewParameter)
{
    int i = GetListIndexOfParameterWithIdx(NewParameter.Index);
    if(i != -1){
        ParametersList[i] = NewParameter;
    }else{
        ParametersList.append(NewParameter);
    }
}

void GrblSettingsDialog::ParseParameterString(QString ParamStr)
{
    QRegExp RexExp;
    unsigned char Idx = 0;
    double Val = 0;
    QString DescStr;

    RexExp = QRegExp("\\$[0-9]+=");
    if(RexExp.indexIn(ParamStr) != -1){
        QString FoundStr(RexExp.cap(0));
        FoundStr = FoundStr.mid(1,FoundStr.length()-2);
        Idx =  (unsigned char)FoundStr.toInt();

        RexExp = QRegExp("=[0-9]*\\.?[0-9]*");
        if(RexExp.indexIn(ParamStr) != -1){
            QString FoundStr(RexExp.cap(0));
            FoundStr = FoundStr.mid(1,FoundStr.length()-1);
            Val =  FoundStr.toDouble();

            RexExp = QRegExp("[(][^)]*[)]");
            if(RexExp.indexIn(ParamStr) != -1){
                QString FoundStr(RexExp.cap(0));
                DescStr = FoundStr.mid(1,FoundStr.length()-2);

                AddParameterToList(GrblParameter(Idx,Val,DescStr));
                if(Idx == PARA_IDX_MAX){refreshList();}
            }
        }
    }
}

void GrblSettingsDialog::refreshList()
{
    disconnect(ui->tableWidgetParameter,SIGNAL(cellChanged(int,int)),this,SLOT(cellChanged(int,int)));

    ui->tableWidgetParameter->clear();
    ui->tableWidgetParameter->setRowCount(0);

    ui->tableWidgetParameter->setHorizontalHeaderLabels(QStringList() << tr("Index") << tr("Value") << tr("Description"));

    ui->tableWidgetParameter->setColumnCount(3);
    ui->tableWidgetParameter->setColumnWidth(0, 35);
    ui->tableWidgetParameter->setColumnWidth(1, 100);
    ui->tableWidgetParameter->setColumnWidth(2, 300);

    for(int i = 0; i < ParametersList.size(); i++){
        ui->tableWidgetParameter->insertRow(ui->tableWidgetParameter->rowCount());

        QTableWidgetItem *IndexWidget = new QTableWidgetItem(QString::number(ParametersList.at(i).Index));
        IndexWidget->setFlags(Qt::NoItemFlags);
        ui->tableWidgetParameter->setItem(i,0,IndexWidget);

        ui->tableWidgetParameter->setItem(i,1,new QTableWidgetItem(QString::number(ParametersList.at(i).Value)));

        QTableWidgetItem *DescWidget = new QTableWidgetItem(ParametersList.at(i).Description);
        DescWidget->setFlags(Qt::NoItemFlags);
        ui->tableWidgetParameter->setItem(i,2,DescWidget);

    }
    connect(ui->tableWidgetParameter,SIGNAL(cellChanged(int,int)),this,SLOT(cellChanged(int,int)));
}
