#include "helicalholecutter.h"
#include "ui_helicalholecutter.h"
#include <QtMath>
#include <QDebug>
#include <QClipboard>

HelicalHoleCutter::HelicalHoleCutter(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::HelicalHoleCutter)
{
    ui->setupUi(this);
}

HelicalHoleCutter::~HelicalHoleCutter()
{
    delete ui;
}


QString HelicalHoleCutter::FormatedValue(qreal Val){
    return QString::number(Val,'f',4);
}


QString HelicalHoleCutter::PrintPoint(QPointF Point, char X_Suffix, char Y_Suffix){

    return QString(X_Suffix +FormatedValue(Point.rx())+ " " +  Y_Suffix + FormatedValue(Point.ry()));
}



void HelicalHoleCutter::on_GenerateButton_clicked()
{
    ui->GCodeTextEdit->clear();
    if(VerifyInputs() == false){
        return;
     }



    qreal X = ui->PosXdoubleSpinBox->value();
    qreal Y = ui->PosYdoubleSpinBox->value();
    qreal D = ui->DiameterdoubleSpinBox->value();
    qreal D_Tool = ui->ToolDdoubleSpinBox->value();
    qreal depth = -ui->DepthdoubleSpinBox->value();
    qreal start = -ui->StartLeveldoubleSpinBox->value();
    qreal feed = ui->IncrementLeveldoubleSpinBox->value();
    int F = ui->FeedRatespinBox->value();
    qreal saveZ = ui->SaftyHeightspinBox->value();

    double ToolCenterRadius = (D-D_Tool)/2;

    QPointF Center(X,Y);

    QPointF CirclePos[3];
    QPointF CircleIJ[3];

    qreal Angle = 0;
    for(int i = 0; i < 3; i++){
        CirclePos[i] = QPointF(ToolCenterRadius*qCos(Angle),ToolCenterRadius*qSin(Angle))+Center;

        qreal AngleIncAbs = qDegreesToRadians(360.0f/3);

        Angle += ui->radioButtonCW->isChecked() ? -AngleIncAbs: AngleIncAbs;
    }

    for(int i = 0; i < 3; i++){
        CircleIJ[i] = Center-CirclePos[(i+2)%3];
    }


    qreal Z = start+0.5;

    int i = 0;

    QString Comment = QString("(Helix: X=%1 Y=%2 D=%3 T=%4)").arg(FormatedValue(X)).arg(FormatedValue(Y)).arg(FormatedValue(D)).arg(FormatedValue(-depth));
     ui->GCodeTextEdit->appendPlainText(Comment);

    if(ui->checkBoxSafetyHeightStart->isChecked()){
        ui->GCodeTextEdit->appendPlainText("G0 Z" + FormatedValue(saveZ));
    }
    ui->GCodeTextEdit->appendPlainText("G0 " + PrintPoint(CirclePos[i], 'X', 'Y'));
    ui->GCodeTextEdit->appendPlainText("G0 Z" + FormatedValue(Z));

    QString GCode = ui->radioButtonCW->isChecked() ? "G2 ": "G3 ";

    i++;
    bool depthReached = false;
    int BottomSegmentCounter = 0;
    while(1){
        Z-= feed/3;
        if(Z <= depth){
            Z = depth;
            depthReached = true;
        }
        QString Cmd = GCode + PrintPoint(CirclePos[i%3], 'X', 'Y')+ " "
                            + PrintPoint(CircleIJ[i%3], 'I', 'J')
                            + " Z" + FormatedValue(Z);
        if(i == 1){Cmd += QString(" F%1").arg(F);}

        ui->GCodeTextEdit->appendPlainText(Cmd);

        if(depthReached){
            BottomSegmentCounter++;
            if(BottomSegmentCounter >= 4){
                break;
            }
        }
        i++;
    }
    ui->GCodeTextEdit->appendPlainText("G0 Z" + FormatedValue(saveZ));
}

bool HelicalHoleCutter::VerifyInputs()
{
    bool AllOk = true;

    if(ui->ToolDdoubleSpinBox->value() < 0.1){qDebug()<<"ToolD"; AllOk = false;}
    if(ui->DiameterdoubleSpinBox->value() < ui->ToolDdoubleSpinBox->value()){qDebug()<<"Diameter"; AllOk = false;}
    if(ui->DepthdoubleSpinBox->value() < 0.01){qDebug()<<"Depth"; AllOk = false;}
    if(ui->StartLeveldoubleSpinBox->value() >  ui->DepthdoubleSpinBox->value()){qDebug()<<"StartLevel"; AllOk = false;}

    return AllOk;
}

void HelicalHoleCutter::on_pushButtonCopy_clicked()
{
    QClipboard *clipboard = QApplication::clipboard();
    clipboard->setText(ui->GCodeTextEdit->toPlainText());
}
