#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "codeeditor.h"
#include <QFile>
#include <QFileDialog>

const int FEED_Z_MAX = 500;

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),GCodeFileLineCount(0), FileOkCounter(0), SettingsDiag(this), ProbeDiag(this),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    SpindeIsOn = false;

    ui->ConnectionLED->setColor(Qt::red);
    connect(&ComModul,SIGNAL(LineReceived(QString)),this,SLOT(ParseReceivedLine(QString)));
    connect(&Tick1000msTimer,SIGNAL(timeout()),this,SLOT(Tick1000ms()));
    Tick1000msTimer.setSingleShot(false);
    Tick1000msTimer.start(1000);

    connect(&ContiniuosJogTimer,SIGNAL(timeout()),this,SLOT(JogTimerTick()));
    ContiniuosJogTimer.setSingleShot(false);
    ContiniuosJogTimer.start(200);

    connect(&Gamepad,SIGNAL(ButtonPressed(GamepadButton)),this, SLOT(GamePadButtonPressed(GamepadButton)));

    connect(ui->plainTextEditGCode,SIGNAL(textChanged()),this, SLOT(EditorTextChanged()));

    Preview3DDiag.SetDataPointer(&PreviewData);
}

MainWindow::~MainWindow()
{
    delete ui;
}



void MainWindow::Tick1000ms()
{
    if(!ComModul.isOpen()){
        ui->ConnectionLED->setColor(Qt::red);
        ComModul.ConnectToSerialDevice(0x20A0,0x423F);
        ui->labelGrblStatus->setText("offline");
    }else{
 //       ComModul.Flush();
        QApplication::processEvents();
        ui->ConnectionLED->setColor(Qt::green);
    }
}

void MainWindow::ParseStatusLine(QString Line)
{
    Line = Line.mid(1,Line.length()-2);
    QStringList Params = Line.split(QRegExp("[,:]"));
    ui->labelGrblStatus->setText(Params.value(0,""));
    ProbeDiag.setGRBLStatusString(Params.value(0,""));
    if(ui->labelGrblStatus->text() != "Idle"){
        ui->pushButtonSpindle->setEnabled(false);
        ui->pushButtonOpenProbeDialog->setEnabled(false);
    }else{
         ui->pushButtonSpindle->setEnabled(true);
         ui->pushButtonOpenProbeDialog->setEnabled(true);
    }


    if(Params.value(1,"") == "MPos" && Params.size() >=5){
        ui->lcdNumberMachineX->display(Params.value(2,""));
        ui->lcdNumberMachineY->display(Params.value(3,""));
        ui->lcdNumberMachineZ->display(Params.value(4,""));
    }
    if(Params.value(5,"") == "WPos" && Params.size() >=9){
        ui->lcdNumberWorkX->display(Params.value(6,""));
        ui->lcdNumberWorkY->display(Params.value(7,""));
        ui->lcdNumberWorkZ->display(Params.value(8,""));
    }
}


void MainWindow::AddLineToConsole(QString Line,QColor Color)
{
    QTextCharFormat tf = ui->plainTextEditSerial->currentCharFormat();
    tf.setForeground(QBrush(Color));
    ui->plainTextEditSerial->setCurrentCharFormat(tf);
    ui->plainTextEditSerial->appendPlainText(Line);
}

void MainWindow::ParseReceivedLine(QString ReceivedLine)
{
    if(ReceivedLine.startsWith('<')){
        ParseStatusLine(ReceivedLine);
        if(ui->checkBoxHideStatusMessage->isChecked()){
            return;
        }
    }else if(ReceivedLine.startsWith('$')){ //Paramter Auslesen
        if(ReceivedLine.size()>=2){
            if(ReceivedLine[1]>='0' && ReceivedLine[1]<='9'){
                SettingsDiag.ParseParameterString(ReceivedLine);
            }
        }
    }else if(ReceivedLine == QString("ok")){
        if(GCodeFileLineCount > 0 && GCodeFileLineCount > FileOkCounter){
            FileOkCounter++;
            ui->progressBar->setValue((int)(100*FileOkCounter/GCodeFileLineCount));
        }
    }
    AddLineToConsole(ReceivedLine,Qt::red);
}


void MainWindow::TryToSendLineAndAddToConsoleText(QString Line)
{
    if(ComModul.SendLine(Line)){
        AddLineToConsole(Line, Qt::blue);
    }
}


void MainWindow::on_lineEditCommand_returnPressed()
{
    if(!ui->lineEditCommand->text().isEmpty()){
        TryToSendLineAndAddToConsoleText(ui->lineEditCommand->text());
    }
    ui->lineEditCommand->clear();
}

void MainWindow::on_pushButtonHelpCmd_clicked(){TryToSendLineAndAddToConsoleText("$");}
void MainWindow::on_pushButtonSettingsCmd_clicked(){TryToSendLineAndAddToConsoleText("$$");}
void MainWindow::on_pushButtonParameterCmd_clicked(){TryToSendLineAndAddToConsoleText("$#");}
void MainWindow::on_pushButtonStateCmd_clicked(){TryToSendLineAndAddToConsoleText("$G");}
void MainWindow::on_pushButtonInfoCmd_clicked(){TryToSendLineAndAddToConsoleText("$I");}
void MainWindow::on_pushButtonStartupBlocksCmd_clicked(){TryToSendLineAndAddToConsoleText("$N");}
void MainWindow::on_pushButtonCheckCmd_clicked(){TryToSendLineAndAddToConsoleText("$C");}
void MainWindow::on_pushButtonKillLockCmd_clicked(){TryToSendLineAndAddToConsoleText("$X");}
void MainWindow::on_pushButtonHomingCycleCmd_clicked(){TryToSendLineAndAddToConsoleText("$H");}
void MainWindow::on_pushButtonCycleStartCmd_clicked(){TryToSendLineAndAddToConsoleText("~");}
void MainWindow::on_pushButtonFeedHoldCmd_clicked(){TryToSendLineAndAddToConsoleText("!");}
void MainWindow::on_pushButtonStatusCmd_clicked(){TryToSendLineAndAddToConsoleText("?");}
void MainWindow::on_pushButtonResetGRBLCmd_clicked(){TryToSendLineAndAddToConsoleText(QString(QChar(0x18)));}


void MainWindow::on_pushButtonResetGRBLCmd_2_clicked(){ui->plainTextEditSerial->clear();}

void MainWindow::on_pushButtonZeroAll_clicked(){TryToSendLineAndAddToConsoleText("G92 X0 Y0 Z0");}
void MainWindow::on_pushButtonZeroX_clicked(){TryToSendLineAndAddToConsoleText("G92 X0");}
void MainWindow::on_pushButtonZeroY_clicked(){TryToSendLineAndAddToConsoleText("G92 Y0");}
void MainWindow::on_pushButtonZeroZ_clicked(){TryToSendLineAndAddToConsoleText("G92 Z0");}



QString MainWindow::GetStepString()
{
    if(ui->radioButton100mm->isChecked()){return QString("100");}
    if(ui->radioButton10mm->isChecked()){return QString("10");}
    if(ui->radioButton1mm->isChecked()){return QString("1");}
    if(ui->radioButton01mm->isChecked()){return QString("0.1");}
    if(ui->radioButton001mm->isChecked()){return QString("0.01");}
    if(ui->radioButtonCustom->isChecked()){return QString::number(ui->doubleSpinBoxCustomStep->value());}
    return QString("0.0");
}


void MainWindow::on_pushButtonPlusX_clicked(){JogButtonPressed(X_PLUS,false);}
void MainWindow::on_pushButtonMinusX_clicked(){JogButtonPressed(X_MINUS,false);}
void MainWindow::on_pushButtonPlusY_clicked(){JogButtonPressed(Y_PLUS,false);}
void MainWindow::on_pushButtonMinusY_clicked(){JogButtonPressed(Y_MINUS,false);}
void MainWindow::on_pushButtonPlusZ_clicked(){JogButtonPressed(Z_PLUS,false);}
void MainWindow::on_pushButtonMinusZ_clicked(){JogButtonPressed(Z_MINUS,false);}


void MainWindow::SendJogCommand(QString AxisDirStr, QString DistStr, QString FeedRateStr)
{
    TryToSendLineAndAddToConsoleText("G91");
    TryToSendLineAndAddToConsoleText(QString("G01 %1%2 F%3").arg(AxisDirStr).arg(DistStr).arg(FeedRateStr));
    on_pushButtonStatusCmd_clicked();
}




void MainWindow::JogButtonPressed(GamepadButton Button, bool WasGamepad)
{
    if((!WasGamepad && ui->radioButtonContinious->isChecked() == false)||(WasGamepad && Gamepad.ButtonState(DEFINED_STEP))){
        QString AxisDirStr = "";

        switch(Button){
            case X_PLUS: AxisDirStr = "X+"; break;
            case X_MINUS: AxisDirStr = "X-"; break;
            case Y_PLUS: AxisDirStr = "Y+"; break;
            case Y_MINUS: AxisDirStr = "Y-"; break;
            case Z_PLUS: AxisDirStr = "Z+"; break;
            case Z_MINUS: AxisDirStr = "Z-"; break;
            default: break;
        }
        if(!AxisDirStr.isEmpty()){
            int FeedRate = ui->spinBoxFeedRate->value();
            if(AxisDirStr.startsWith("Z") && FeedRate > FEED_Z_MAX){
                FeedRate = FEED_Z_MAX;
            }
            SendJogCommand(AxisDirStr,GetStepString(),QString::number(FeedRate));
        }
    }
    switch(Button){
        case ZERO: on_pushButtonZeroAll_clicked(); break;
        case START: on_pushButtonSendGCode_clicked(); break;
        case SPINDLE: on_pushButtonSpindle_clicked(); break;
        default: break;
    }
}


void MainWindow::GamePadButtonPressed(GamepadButton Button)
{
    JogButtonPressed(Button, true);
}


void MainWindow::JogTimerTick()
{
    QString AxisDirStr = "";

    if(ui->radioButtonContinious->isChecked()){
        if(ui->pushButtonPlusX->isDown()){AxisDirStr = "X+";}
        else if(ui->pushButtonMinusX->isDown()){AxisDirStr = "X-";}
        else if(ui->pushButtonPlusY->isDown()){AxisDirStr = "Y+";}
        else if(ui->pushButtonMinusY->isDown()){AxisDirStr = "Y-";}

        else if(ui->pushButtonPlusZ->isDown()){AxisDirStr = "Z+";}
        else if(ui->pushButtonMinusZ->isDown()){AxisDirStr = "Z-";}
    }


    if(!Gamepad.ButtonState(DEFINED_STEP)){
            if(Gamepad.ButtonState(X_PLUS)){AxisDirStr = "X+";}
            else if(Gamepad.ButtonState(X_MINUS)){AxisDirStr = "X-";}
            else if(Gamepad.ButtonState(Y_PLUS)){AxisDirStr = "Y+";}
            else if(Gamepad.ButtonState(Y_MINUS)){AxisDirStr = "Y-";}

            else if(Gamepad.ButtonState(Z_PLUS)){AxisDirStr = "Z+";}
            else if(Gamepad.ButtonState(Z_MINUS)){AxisDirStr = "Z-";}
    }
    if(AxisDirStr.isEmpty()){return;}

    int FeedRate = ui->spinBoxFeedRate->value();

    if(Gamepad.ButtonState(SLOW)){FeedRate = 50;}
    else if(Gamepad.ButtonState(MEDIUM_SPEED)){FeedRate = 300;}

    if(AxisDirStr.startsWith("Z") && FeedRate > FEED_Z_MAX){
        FeedRate = FEED_Z_MAX;
    }
    QString StepDistStr = QString::number(((double)FeedRate)/(5*60),'f',3);

    SendJogCommand(AxisDirStr, StepDistStr, QString::number(FeedRate));
}

void MainWindow::EditorTextChanged()
{
    ParseGcode();
    Preview3DDiag.Update();
}



void MainWindow::on_spinBoxFeedRate_editingFinished()
{
    ui->verticalSliderFeedRate->setValue(ui->spinBoxFeedRate->value()/10);
}


void MainWindow::on_pushButtonSendGCode_clicked()
{
    if(ui->labelGrblStatus->text() == "Idle"){
        QStringList GCodeLines = ui->plainTextEditGCode->toPlainText().split(QRegExp("[\r\n]"),QString::SkipEmptyParts);
        GCodeFileLineCount = GCodeLines.size();
        FileOkCounter = 0;
        foreach(QString str, GCodeLines)
        {
           TryToSendLineAndAddToConsoleText(str);
        }
    }
}


void MainWindow::on_pushButtonOpenFile_clicked()
{
    QString LoadFileName = QFileDialog::getOpenFileName(this, "Open", QDir::currentPath(),"GCode (*.nc)");
    if(LoadFileName.isEmpty()){return;}
    QFile LoadFile(LoadFileName);
    if(!LoadFile.open(QIODevice::ReadOnly)){return;}
    ui->plainTextEditGCode->setPlainText(LoadFile.readAll());
    LoadFile.close();
}

void MainWindow::on_pushButtonOpenParaConfig_clicked()
{
   on_pushButtonSettingsCmd_clicked();
   SettingsDiag.show();
}


void MainWindow::on_verticalSliderFeedRate_valueChanged(int value)
{
    ui->spinBoxFeedRate->setValue(value*10);
}

void MainWindow::on_pushButtonSpindle_clicked()
{
    if(ui->pushButtonSpindle->isEnabled()){
        SetSpindle(!SpindeIsOn);
    }
}

void MainWindow::SetSpindle(bool TurnOn)
{
    if(TurnOn){
        TryToSendLineAndAddToConsoleText("M3");
        SpindeIsOn = true;
        ui->pushButtonSpindle->setText("Spindle Off");
    }else{
        TryToSendLineAndAddToConsoleText("M5");
        SpindeIsOn = false;
        ui->pushButtonSpindle->setText("Spindle On");
    }
}

void MainWindow::on_pushButtonOpenProbeDialog_clicked()
{
    ProbeDiag.show();
}

void MainWindow::on_pushButton3DPreview_clicked()
{
    Preview3DDiag.show();
    ParseGcode();
    Preview3DDiag.Update();
}

void MainWindow::on_pushButtonHHC_clicked()
{
    HHCDiag.show();
}
