#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QTimer>
#include <QList>
#include "terminalcommunication.h"
#include "grblsettingsdialog.h"
#include "probingdialog.h"
#include "preview3d.h"
#include "windowsgamepad.h"
#include <QVector4D>
#include "helicalholecutter.h"

namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

    void TryToSendLineAndAddToConsoleText(QString Line);

private slots:

    void ParseReceivedLine(QString ReceivedLine);
    void Tick1000ms();
    void JogTimerTick();
    void EditorTextChanged();
    void GamePadButtonPressed(GamepadButton Button);
    void JogButtonPressed(GamepadButton Button, bool WasGamepad);

    void on_lineEditCommand_returnPressed();

    void on_pushButtonHelpCmd_clicked();

    void on_pushButtonSettingsCmd_clicked();

    void on_pushButtonParameterCmd_clicked();

    void on_pushButtonStateCmd_clicked();

    void on_pushButtonInfoCmd_clicked();

    void on_pushButtonStartupBlocksCmd_clicked();

    void on_pushButtonCheckCmd_clicked();

    void on_pushButtonKillLockCmd_clicked();

    void on_pushButtonHomingCycleCmd_clicked();

    void on_pushButtonCycleStartCmd_clicked();

    void on_pushButtonFeedHoldCmd_clicked();

    void on_pushButtonStatusCmd_clicked();

    void on_pushButtonResetGRBLCmd_clicked();

    void on_pushButtonResetGRBLCmd_2_clicked();

    void on_pushButtonZeroAll_clicked();

    void on_pushButtonZeroX_clicked();

    void on_pushButtonZeroY_clicked();

    void on_pushButtonZeroZ_clicked();

    void on_pushButtonMinusX_clicked();

    void on_pushButtonPlusX_clicked();

    void on_pushButtonPlusY_clicked();

    void on_pushButtonMinusY_clicked();

    void on_pushButtonPlusZ_clicked();

    void on_pushButtonMinusZ_clicked();

    void on_verticalSliderFeedRate_valueChanged(int value);

    void on_spinBoxFeedRate_editingFinished();

    void on_pushButtonSendGCode_clicked();

    void on_pushButtonOpenFile_clicked();

    void on_pushButtonOpenParaConfig_clicked();


    void on_pushButtonSpindle_clicked();

    void on_pushButtonOpenProbeDialog_clicked();

    void on_pushButton3DPreview_clicked();

    void on_pushButtonHHC_clicked();

private:

    bool SpindeIsOn;

    void SetSpindle(bool TurnOn);

    void SendJogCommand(QString AxisDirStr, QString DistStr, QString FeedRateStr);
    void AddLineToConsole(QString Line, QColor Color);
    void ParseStatusLine(QString Line);


    QVector<QVector4D>PreviewData;
    void ParseGcode(void);

    int GCodeFileLineCount, FileOkCounter;

    QString GetStepString();

    GrblSettingsDialog SettingsDiag;
    ProbingDialog ProbeDiag;
    Preview3D Preview3DDiag;
    HelicalHoleCutter HHCDiag;

    Ui::MainWindow *ui;

    TerminalCommunication  ComModul;
    QTimer Tick1000msTimer;
    QTimer ContiniuosJogTimer;

    WindowsGamepad Gamepad;
};

#endif // MAINWINDOW_H
