#include "windowsgamepad.h"
#include <QDebug>


#define BTN_IDX_Z_PLUS 4
#define BTN_IDX_Z_MINUS 5
#define BTN_IDX_ZERO 2
#define BTN_IDX_SLOW 1
#define BTN_IDX_MEDIUM_SPEED 0
#define BTN_IDX_DEFINED_STEP 3
#define BTN_IDX_START 9
#define BTN_IDX_SPINDLE 8


WindowsGamepad::WindowsGamepad(QObject *parent) :
    QObject(parent),GamePadId(-1)
{

    for(int i = 0; i < BUTTON_COUNT; i++){LastState.Buttons[i] = false;} 

    connect(&UpdateTickTimer,SIGNAL(timeout()),this,SLOT(UpdateTickSlot()));
    UpdateTickTimer.setSingleShot(false);
    UpdateTickTimer.start(50);
}


void WindowsGamepad::FindGamepad()
{
    unsigned int maxGamepadId = joyGetNumDevs();
  //   qDebug()<<"MaxID"<<maxGamepadId;
    for (unsigned int id = 0; id < maxGamepadId; ++id) {
        info.dwSize = sizeof(info);
        MMRESULT result = joyGetPosEx(id, &info);
   //     qDebug()<<"result"<<result;
        bool isConnected = (result == JOYERR_NOERROR);
        if(isConnected){
            GamePadId = id;
      //      qDebug() << "gamepad #" << GamePadId << "connected";
        }
    }
}


void WindowsGamepad::UpdateTickSlot()
{
    if(GamePadId == -1){
        FindGamepad();
        return;
    }
    info.dwFlags = JOY_RETURNBUTTONS | JOY_RETURNX | JOY_RETURNY;
    if(joyGetPosEx(GamePadId, &info) == JOYERR_NOERROR){

        GamepadState CurrentState;
        QString TestStr;

        for(int i = 0; i < BUTTON_COUNT; i++){CurrentState.Buttons[i]=false;}

        if(info.dwXpos > 32511){CurrentState.Buttons[X_PLUS] = true;}
        if(info.dwXpos < 32511){CurrentState.Buttons[X_MINUS] = true;}
        if(info.dwYpos < 32511){CurrentState.Buttons[Y_PLUS] = true;}
        if(info.dwYpos > 32511){CurrentState.Buttons[Y_MINUS] = true;}

        if(info.dwButtons & (1<<BTN_IDX_Z_PLUS)){CurrentState.Buttons[Z_PLUS] = true;}
        if(info.dwButtons & (1<<BTN_IDX_Z_MINUS)){CurrentState.Buttons[Z_MINUS] = true;}
        if(info.dwButtons & (1<<BTN_IDX_ZERO)){CurrentState.Buttons[ZERO] = true;}
        if(info.dwButtons & (1<<BTN_IDX_SLOW)){CurrentState.Buttons[SLOW] = true;}
        if(info.dwButtons & (1<<BTN_IDX_MEDIUM_SPEED)){CurrentState.Buttons[MEDIUM_SPEED] = true;}
        if(info.dwButtons & (1<<BTN_IDX_START)){CurrentState.Buttons[START] = true;}
        if(info.dwButtons & (1<<BTN_IDX_SPINDLE)){CurrentState.Buttons[SPINDLE] = true;}
        if(info.dwButtons & (1<<BTN_IDX_DEFINED_STEP)){CurrentState.Buttons[DEFINED_STEP] = true;}

        for(int i = 0; i < BUTTON_COUNT; i++){
            if(CurrentState.Buttons[i] != LastState.Buttons[i]){
                if(CurrentState.Buttons[i] == true){
                    emit ButtonPressed((GamepadButton)i);
                }else{
                    emit ButtonReleased((GamepadButton)i);
                }
            }
        }

        LastState = CurrentState;
    }else{
        qDebug() << "gamepad #" << GamePadId << "disconnected";
        GamePadId = -1;
    }
}

